GridStatusEmphasize.Modes = { }

local temp_emphasized_units = GridStatusEmphasize.temp_emphasized_units
local static_emphasized_units = GridStatusEmphasize.static_emphasized_units
local core = GridStatusEmphasize.core
local GridRoster = Grid:GetModule("GridRoster")
local settings

local cos = math.cos
local GetTime = GetTime

function GridStatusEmphasize.Modes:Initialize()
	settings = GridStatusEmphasize.db.profile.alert_emphasize
end

local function tisempty(t)
	return next(t) == nil
end

-- Lower
GridStatusEmphasize.Modes.Lower = {}

function GridStatusEmphasize.Modes.Lower:Initialize()
end

function GridStatusEmphasize.Modes.Lower:Disable()
end

function GridStatusEmphasize.Modes.Lower:Update()
	if tisempty(temp_emphasized_units) then
		core:SendStatusLostAllUnits("alert_emphasize")
	else
		for guid, unitid in GridRoster:IterateRoster() do
			if temp_emphasized_units[guid] or static_emphasized_units[guid] then
				core:SendStatusLost(guid, "alert_emphasize")
			else
				core:SendStatusGained(guid,
							"alert_emphasize",
							settings.priority,
							nil,
							{r=0, g=0, b=0, a=settings.others_alpha},
							1,
							nil,
							nil)
			end
		end
	end
end

-- Smooth blink
GridStatusEmphasize.Modes.SmoothBlink = {}

function GridStatusEmphasize.Modes.SmoothBlink:Initialize()
	if not self.update_frame then
		self.update_frame = CreateFrame("Frame")		
	end
	
	self.update_frame:SetScript("OnUpdate", function(_, elapsed) GridStatusEmphasize.Modes.SmoothBlink:BlinkEmphasizeStep(elapsed) end)
end

function GridStatusEmphasize.Modes.SmoothBlink:Disable()
	self.update_frame:SetScript("OnUpdate", nil)
end

function GridStatusEmphasize.Modes.SmoothBlink:Update()
	if tisempty(temp_emphasized_units) then
		core:SendStatusLostAllUnits("alert_emphasize")
	else
		for guid, unitid in GridRoster:IterateRoster() do
			if not temp_emphasized_units[guid] then
				core:SendStatusLost(guid, "alert_emphasize")
			end
		end
	end
end

local cycle = 2*math.pi
local alpha_low = 0.3

local update_time = 0

function GridStatusEmphasize.Modes.SmoothBlink:BlinkEmphasizeStep(elapsed)
	update_time = update_time + elapsed
	
	if update_time >= (1/30) then
		update_time = 0
		
		if not tisempty(temp_emphasized_units) then
			local x = GetTime() * cycle * settings.cycles_sec
			local alpha = (cos(x) + 1) * 0.5 * (1 - alpha_low) + alpha_low
			
			for guid, tags in pairs(temp_emphasized_units) do
				core:SendStatusGained(guid,
							"alert_emphasize",
							settings.priority,
							nil,
							{r=0, g=0, b=0, a=alpha},
							1,
							nil,
							nil)
			end
		end
	end
end
