local Emphasize = Grid:GetModule("GridStatus"):GetModule("GridStatusEmphasize")
local QuickHealth = LibStub and LibStub("LibQuickHealth-2.0", true) -- don't error if not found
local GridStatusEmphasize_Health = Emphasize:NewModule("GridStatusEmphasize_Health", "AceEvent-3.0")

local GridRoster = Grid:GetModule("GridRoster")
local GridFrame = Grid:GetModule("GridFrame")

-- upvalues
local UnitHealth = QuickHealth and QuickHealth.UnitHealth or UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitGUID = UnitGUID
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitIsConnected = UnitIsConnected
local UnitIsCharmed = UnitIsCharmed
local UnitIsEnemy = UnitIsEnemy
local UnitIsPlayer = UnitIsPlayer

-- data
local SPELLID_SOR = 27827 -- spirit of redemption

local spellname_sor = "Not Yet Known" -- just a temporary name until the spellname is updated, to prevent UnitBuff errors

local settings


GridStatusEmphasize_Health.defaultDB = {
	always_show_lowhp_pc = 0.5,
	emphasize_lowhp_pc = 0.0,
	range = true,
	ignore_pets = true,
}

local options = {
	["lowhp_pc"] = {
		order = 1,
		type = "range",
		name = "Never fade out threshold",
		desc = "Never fade out units under HP % (0 to disable)",
		get = function () return GridStatusEmphasize_Health.db.profile.always_show_lowhp_pc*100 end,
		set = function (_, v)
				GridStatusEmphasize_Health.db.profile.always_show_lowhp_pc = v/100
				GridStatusEmphasize_Health:UpdateAll()
			end,
		min = 0,
		max = 100,
		step = 1,
	},
	["emphasize_lowhp_pc"] = {
		order = 2,
		type = "range",
		name = "Emphasize threshold",
		desc = "Emphasize units under HP % (0 to disable)",
		get = function () return GridStatusEmphasize_Health.db.profile.emphasize_lowhp_pc*100 end,
		set = function (_, v)
				GridStatusEmphasize_Health.db.profile.emphasize_lowhp_pc = v/100
				GridStatusEmphasize_Health:EnableDisable()
			end,
		min = 0,
		max = 100,
		step = 1,
	},
	["ignore_pets"] = {
		order = 3,
		type = "toggle",
		name = "Ignore pets",
		desc = "Don't emphasize pets, even when they are below the threshold",
		get = function () return GridStatusEmphasize_Health.db.profile.ignore_pets end,
		set = function (_, v)
				GridStatusEmphasize_Health.db.profile.ignore_pets = v
				GridStatusEmphasize_Health:UpdateAll()
			end,
	},
	["range"] = {
		order = 4,
		type = "toggle",
		name = "Range filter",
		desc = "Don't emphasize units out of range, even when they are below the threshold",
		get = function () return GridStatusEmphasize_Health.db.profile.range end,
		set = function (_, v)
				GridStatusEmphasize_Health.db.profile.range = v
				GridStatusEmphasize_Health:EnableDisable()
			end,
	},
}

function GridStatusEmphasize_Health:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterOptions("Health", options)
	settings = self.db.profile
	
	self:RegisterEvent("PLAYER_LOGIN", "UpdateSpellInfo")
	
	self:EnableDisable()
end

function GridStatusEmphasize_Health:UpdateSpellInfo()
	spellname_sor = GetSpellInfo(SPELLID_SOR)
end

function GridStatusEmphasize_Health:EnableDisable()
	if settings.always_show_lowhp_pc ~= 0 or settings.emphasize_lowhp_pc ~= 0 then
		if QuickHealth then
			QuickHealth.RegisterCallback(self, "UnitHealthUpdated", "UnitHealthUpdated")
		else
			self:RegisterEvent("UNIT_HEALTH", "UpdateUnit")
		end

		self:RegisterMessage("Grid_UnitJoined")
		self:RegisterEvent("UNIT_HEALTH_MAX", "UpdateUnit")
		self:RegisterEvent("UNIT_CONNECTION", "UpdateUnit")
		self:RegisterEvent("UNIT_NAME_UPDATE", "UpdateUnit")
		self:RegisterEvent("RAID_ROSTER_UPDATE", "UpdateAll")
		self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateAll")
		self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAll")
	else
		if QuickHealth then
			QuickHealth.UnregisterCallback(self, "UnitHealthUpdated")
		else
			self:UnregisterEvent("UNIT_HEALTH")
		end
		self:UnregisterMessage("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_HEALTH_MAX")
		self:UnregisterEvent("UNIT_CONNECTION")
		self:UnregisterEvent("UNIT_NAME_UPDATE")
		self:UnregisterEvent("RAID_ROSTER_UPDATE")
		self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end	

	self:UpdateAll()
end

function GridStatusEmphasize_Health:Grid_UnitJoined(event, guid, unitid)
	self:UpdateUnit("Grid_UnitJoined", unitid)
end

function GridStatusEmphasize_Health:UpdateAll()
	Emphasize:Pause()
	
	Emphasize:StaticDeemphasizeAllUnits("lowhp")
	Emphasize:TempDeemphasizeAllUnits("lowhp")
	
	for guid, unitid in GridRoster:IterateRoster() do
		self:UpdateUnit("UpdateAll", unitid)
	end
	
	Emphasize:Resume()
end

function GridStatusEmphasize_Health:UnitHealthUpdated(unitid, health, healthmax)
	local guid = UnitGUID(unitid)
	
	if GridRoster:IsGUIDInRaid(guid) then
		if UnitIsDeadOrGhost(unitid) or 
			(not UnitIsConnected(unitid)) or 
			UnitIsCharmed(unitid) or 
			UnitIsEnemy("player", unitid) or
			UnitBuff(unitid, spellname_sor)	then
			Emphasize:TempDeemphasizeUnit(guid, "lowhp")
			Emphasize:StaticDeemphasizeUnit(guid, "lowhp")
		else
		--if not UnitIsDeadOrGhost(unitid) and UnitIsConnected(unitid) and not (UnitIsCharmed(unitid) and UnitIsEnemy("player", unitid)) then
			local hp = health / healthmax
			
			if (settings.emphasize_lowhp_pc > 0) and (hp < settings.emphasize_lowhp_pc) and (not settings.range or GridFrame:UnitInRange(unitid, 40)) and (not settings.ignore_pets or UnitIsPlayer(unitid)) then
				Emphasize:TempEmphasizeUnit(guid, "lowhp")
			else
				Emphasize:TempDeemphasizeUnit(guid, "lowhp")
			end

			if (settings.always_show_lowhp_pc > 0) and (hp <  settings.always_show_lowhp_pc) then
				Emphasize:StaticEmphasizeUnit(guid, "lowhp")
			else
				Emphasize:StaticDeemphasizeUnit(guid, "lowhp")
			end
		end
	end
end

function GridStatusEmphasize_Health:UpdateUnit(event, unitid)
	self:UnitHealthUpdated(unitid, UnitHealth(unitid), UnitHealthMax(unitid))
end
